/* Rev:$Revision: 1.0 $ */
/******************************************************************************
 * Copyright 2006 NetBurner, Inc.  ALL RIGHTS RESERVED
 *   Permission is hereby granted to purchasers of NetBurner Hardware
 *   to use or modify this computer program for any use as long as the
 *   resultant program is only executed on NetBurner provided hardware.
 *
 *   No other rights to use this program or it's derivitives in part or
 *   in whole are granted.
 *
 *   It may be possible to license this or other NetBurner software for
 *   use on non NetBurner Hardware. Please contact Licensing@Netburner.com 
 *   for more information.
 *
 *   NetBurner makes no representation or warranties with respect to the 
 *   performance of this computer program, and specifically disclaims any 
 *   responsibility for any damages, special or consequential, connected 
 *   with the use of this program.
 *
 *   NetBurner, Inc
 *   5405 Morehouse Drive
 *   San Diego Ca, 92121
 *   http://www.netburner.com
 *
 *****************************************************************************/
/******************************************************************************
 Macromedia Flash Demo Program
 This is a support file for the Flash demo that handles the connection 
 between the flash application and the NetBurner device.
*****************************************************************************/
#include <predef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <startnet.h>
#include <tcp.h>

#define TCP_LISTEN_PORT (10000) 				
#define RX_BUFSIZE (5000)
#define TX_BUFSIZE (5000)		


char RXBuffer[RX_BUFSIZE]; 
char TXBuffer[TX_BUFSIZE]; 
char cross_domain_policy[] = "<?xml version=\"1.0\"?> <!DOCTYPE cross-domain-policy SYSTEM \"http://www.adobe.com/xml/dtds/cross-domain-policy.dtd\"> <cross-domain-policy> <site-control permitted-cross-domain-policies=\"all\"/> <allow-access-from domain=\"*\" to-ports=\"*\" secure=\"false\"/> <allow-http-request-headers-from domain=\"*\" headers=\"*\"  secure=\"false\"/> </cross-domain-policy>\0";
#define NFDS (10)	// Max number of file descriptors/connections 

int fd_array[NFDS]; // Array of TCP file descriptors
int valid_array[NFDS]; // We've recieved a heartbeat request from this host.


extern float direction;
extern float speed;
extern int freq;

BOOL Connected = FALSE;

/*-------------------------------------------------------------------
 Function to display connected message
 ------------------------------------------------------------------*/
void DisplayConnectedMsg( int sock, PCSTR url )
{
	writestring(sock, "<html><body><h1>Error, a client is already connected</h1></body></html>");
}


/*-------------------------------------------------------------------
 Send Status of Leds & DIP Switches to Flash Page
 -------------------------------------------------------------------*/
void SendStatus(int fdnet)
{
	float lDirection= direction;
	float lSpeed = speed;
	int lFreq = freq;
	// Send data to web page
	char 	status_string[40];
	
	
    sprintf( status_string, "$|%3.2f|%3.2f|%d", lDirection, lSpeed,lFreq );
	write( fdnet, status_string, strlen( status_string ) + 1 );
}


/*-------------------------------------------------------------------
  Network connection handler
 -------------------------------------------------------------------*/
void ConnectionServer( void * pd )
{
    int     result=0;

  /*
     Listen for incoming TCP connections. You only need to call 
     listen() one time.  
     - Any IP address
     - Port number = PORT
     - Queue up to 5 incoming connection requests
    */
   int fdl = listen( 0, TCP_LISTEN_PORT, 10 );
   iprintf( "Listening for incoming connections on port %d\r\n", TCP_LISTEN_PORT );

	while ( 1 )
	{
		      // Declare file descriptor sets for select() 
		      fd_set read_fds;
		      fd_set error_fds;
		
		      // Init the fd sets
		      FD_ZERO( &read_fds );
		      FD_ZERO( &error_fds );
		      
				// Push data to the host if we have a valid File descriptor
				// and the host has sent us a heartbeat
				// if we got a heartbeat then the cross domain policy file is sent 
	      		for ( int i = 0; i < NFDS; i++ )
	      		{
	         		if ( fd_array[i] && valid_array[i] )
	            	{
	            		SendStatus(fd_array[i]);
	            	}
	      		}
	
		      // Configure the fd sets so select() knows what to process. In
		      // this case any fd data to be read, or an error.
		      for ( int i = 0; i < NFDS; i++ )
		      {
		         if ( fd_array[i] )
		         {
		            FD_SET( fd_array[i], &read_fds );
		            FD_SET( fd_array[i], &error_fds );
		         }
		      }
		
		      // select() should also process the listen fd
		      FD_SET( fdl, &read_fds );
		      FD_SET( fdl, &error_fds );
		
		      /*
		        select() will block until any fd has data to be read, or 
		        has an error. When select() returns, read_fds and/or
		        error_fds variables will have been modified to reflect
		        the events. 
		       */ 
		      select( FD_SETSIZE, &read_fds, ( fd_set * )0, &error_fds, 0 );
		
		      // If the listen fd has a connection request, accept it.
		      if ( FD_ISSET( fdl, &read_fds ) )
		      {
					        IPADDR client_ip;
					        WORD   client_port;
		
		         			int fda = accept( fdl, &client_ip, &client_port, 0 );
		
					         // If accept() succeeded, find an open fd array slot
					         if ( fda > 0 )
					         {
					            for ( int i = 0; i < NFDS; i++ )
					            {
					               if ( fd_array[i] == 0 )
					               {
					                  fd_array[i] = fda;
					                  valid_array[i]=0;
					                  //writestring( fda, "Welcome! 'Q' to quit." );
					                  iprintf( "Added connection on fd[%d] = %d, ", i, fda );
					                  ShowIP(client_ip);
					                  iprintf(":%d\n", client_port);
					                  fda = 0;
					                  break;
					               }
					            }
					         }
		
					         // If no array positions are open, close the connection
					         if ( fda )
					         {
					            writestring( fda, "Server Full" );
					         	iprintf("Server Full\r\n");
					            close( fda );
					         }
		      }
		
		      // If the listen fd has an error, close it and reopen
		      if ( FD_ISSET( fdl, &error_fds ) )
		      {
		         close( fdl );
		         fdl = listen( 0, TCP_LISTEN_PORT, 10 );
		      }
		
		      // Process each fd array element and check it against read_fds
		      // and error_fds. 
		      for ( int i = 0; i < NFDS; i++ )
		      {
		         if ( fd_array[i] )
		         {
					            // Check for data to be read
					            if ( FD_ISSET( fd_array[i], &read_fds ) )
					            {
					            
					               int rv = read( fd_array[i], RXBuffer, RX_BUFSIZE );
								  
			
					               
					               if ( rv > 0 )
					               {
						                  RXBuffer[rv] = 0;
						                      
							            // Reply to policy file request
							            if( strcmp( "<policy-file-request/>", RXBuffer ) == 0 )
							            {
							            	iprintf( "Cross domain sent to %d" ,i );
							            	writeall( fd_array[i], cross_domain_policy, strlen(cross_domain_policy));
								     		close(fd_array[i]);
								     		fd_array[i] = 0;
								     		valid_array[i]=0;
								     		break;
										}		
					               
						                // Process the 8-bit command
						                switch ( RXBuffer[0] ) 
						                {
						                	case '~':			                    // heartbeat signal
						    					write( fd_array[i], "!", 2 );
						    					valid_array[i]=1;
						    					break;
						
						                	case '$': 								// update leds command
						                        RXBuffer[0] = '0';                  // strip out the $ for the atoi
							                    result = atoi(RXBuffer);
						
						                        if( (result >=0) && (result <=8) )  // make sure data is valid
						                            
						                        SendStatus(fd_array[i]);
						                		break;
						
						                	case '?':                 				// send status update
						            	        SendStatus(fd_array[i]);
												break;
						
						                	default:  								// unrecognized data
						                		break;                              
						                }
					            	}
					            }
		            

		
					            // Check for errors
					            if ( FD_ISSET( fd_array[i], &error_fds ) )
					            {
					               iprintf( "Error on fd[%d], closing connection\r\n", i );
					               close( fd_array[i] );
					               valid_array[i]=0;
					               fd_array[i] = 0;
					            }
		         }
		      }
		      

	}
}


